@@ -1,9 +1,9 @@
 #include "apngasm.h"
 #include <cstdlib>
+#include <filesystem>
 #include <png.h>
 #include <zlib.h>
 #include <boost/algorithm/string/predicate.hpp>
-#include <boost/filesystem/operations.hpp>
 #include <boost/regex.hpp>
 #include <boost/range/algorithm.hpp>
 #ifdef APNG_SPECS_SUPPORTED
@@ -56,13 +56,13 @@ namespace apngasm {
     {
       static std::vector<std::string> files;
 
-      boost::filesystem::path nativePath(filepath);
+      std::filesystem::path nativePath(filepath);
       nativePath.make_preferred();
 
       // filepath is current directory.
       if( !nativePath.has_parent_path() )
       {
-        const std::string currentDirPath = "." + std::string(1, boost::filesystem::path::preferred_separator);
+        const std::string currentDirPath = "." + std::string(1, std::filesystem::path::preferred_separator);
         nativePath = currentDirPath + nativePath.string();
       }
 
@@ -76,14 +76,14 @@ namespace apngasm {
         if (!boost::algorithm::iends_with(nativePath.string(), ".png"))
           nativePath = nativePath.string() + ".png";
 
-        if (boost::filesystem::exists(nativePath))
+        if (std::filesystem::exists(nativePath))
           files.push_back(nativePath.string());
       }
 
       // File path has wildcard.
       else
       {
-        const boost::filesystem::path &parentPath = nativePath.parent_path();
+        const std::filesystem::path &parentPath = nativePath.parent_path();
 
         // Convert filepath.
         static const boost::regex escape("[\\^\\.\\$\\|\\(\\)\\[\\]\\+\\?\\\\]");
@@ -93,16 +93,16 @@ namespace apngasm {
         nativePath = boost::regex_replace(nativePath.string(), wildcard, ".*");
 
         // Skip if directory is not found.
-        if (!boost::filesystem::exists(parentPath))
+        if (!std::filesystem::exists(parentPath))
           return files;
 
         // Search files.
         const boost::regex filter(nativePath.string());
-        const boost::filesystem::directory_iterator itEnd;
-        for (boost::filesystem::directory_iterator itCur(parentPath); itCur != itEnd; ++itCur)
+        const std::filesystem::directory_iterator itEnd;
+        for (std::filesystem::directory_iterator itCur(parentPath); itCur != itEnd; ++itCur)
         {
           // Skip if not a file.
-          if (!boost::filesystem::is_regular_file(itCur->status()))
+          if (!std::filesystem::is_regular_file(itCur->status()))
             continue;
 
           // Skip if no match.
@@ -1413,7 +1413,7 @@ namespace apngasm {
           _op[j].valid = 0;
 
         /* dispose = none */
-        get_rect(_width, _height, _frames[n]._pixels, _frames[n+1]._pixels, over1, bpp, rowbytes, zbuf_size, has_tcolor, tcolor, 0);
+        get_rect(_width, _height, _frames[n]._pixels, _frames[n+1]._pixels, over1, coltype, bpp, rowbytes, zbuf_size, has_tcolor, tcolor, 0);
 
         /* dispose = background */
         if (has_tcolor)
@@ -1427,12 +1427,12 @@ namespace apngasm {
             for (j=0; j<h0; j++)
               memset(temp + ((j+y0)*_width + x0)*bpp, tcolor, w0*bpp);
 
-          get_rect(_width, _height, temp, _frames[n+1]._pixels, over2, bpp, rowbytes, zbuf_size, has_tcolor, tcolor, 1);
+          get_rect(_width, _height, temp, _frames[n+1]._pixels, over2, coltype, bpp, rowbytes, zbuf_size, has_tcolor, tcolor, 1);
         }
 
         /* dispose = previous */
         if (n > first)
-          get_rect(_width, _height, prev, _frames[n+1]._pixels, over3, bpp, rowbytes, zbuf_size, has_tcolor, tcolor, 2);
+          get_rect(_width, _height, prev, _frames[n+1]._pixels, over3, coltype, bpp, rowbytes, zbuf_size, has_tcolor, tcolor, 2);
 
         op_min = _op[0].size;
         op_best = 0;
@@ -1733,7 +1733,7 @@ namespace apngasm {
     deflateReset(&_op_zstream2);
   }
 
-  void APNGAsm::get_rect(unsigned int w, unsigned int h, unsigned char *pimage1, unsigned char *pimage2, unsigned char *ptemp, unsigned int bpp, unsigned int stride, int zbuf_size, unsigned int has_tcolor, unsigned int tcolor, int n)
+  void APNGAsm::get_rect(unsigned int w, unsigned int h, unsigned char *pimage1, unsigned char *pimage2, unsigned char *ptemp, unsigned char coltype, unsigned int bpp, unsigned int stride, int zbuf_size, unsigned int has_tcolor, unsigned int tcolor, int n)
   {
     unsigned int   i, j, x0, y0, w0, h0;
     unsigned int   x_min = w-1;
@@ -1759,7 +1759,8 @@ namespace apngasm {
         if (*pa++ != c)
         {
           diffnum++;
-          if (has_tcolor && c == tcolor) over_is_possible = 0;
+          if (coltype == 0 && has_tcolor && c == tcolor) over_is_possible = 0;
+          if (coltype == 3 && _trns[c] != 0xFF) over_is_possible = 0;
           if (i<x_min) x_min = i;
           if (i>x_max) x_max = i;
           if (j<y_min) y_min = j;
